package it.mokabyte.pattern.ejb.session.stateful;

import java.rmi.*;
import javax.ejb.*;
import java.util.*;

import javax.naming.*;
import javax.rmi.PortableRemoteObject;

// Facade to Shopper SF
import it.mokabyte.pattern.ejb.session.stateful.ShopperHome;
import it.mokabyte.pattern.ejb.session.stateful.Shopper;

// Facade to Account BMP
import it.mokabyte.pattern.ejb.entity.AccountHome;
import it.mokabyte.pattern.ejb.entity.Account;
import it.mokabyte.pattern.pk.AccountPK;

import it.mokabyte.pattern.exception.*;
import it.mokabyte.pattern.util.JndiNames;
import it.mokabyte.pattern.util.SimpleTrace;
import it.mokabyte.pattern.valueobject.ProductOM;
import it.mokabyte.pattern.valueobject.AccountOM;


/**
 * <p>Title: FacadeBeanSF </p>
 * <p>Description: it's a Facade to : </p>
 * <li> Stateful   : Shopper
 * <li> Entity BMP : Account
 * <p>Take conversational state with client and store/remove facading to ShopperBean.
 * Make Account Business rules and access the account data facading Account entity bean.</p>
 * <p>Copyright: Copyright (c) 2001</p>
 * <p>Company: </p>
 * @author Rossini S. - L. Dozio
 * @version 1.0
 */
public class FacadeBean implements SessionBean {

  /** Reference to the Shopper stateful bean associated with this bean instance */
  private Shopper shopper = null;

  /** Reference to the Account entity bean associated with this bean instance */
  private Account account = null;

  /** Session Context */
  private SessionContext ctx=null;

  /**
    * Sets up the initial state of the facade bean, lookup the EJB to be facaded.
    * @param ID the ID for the customer
    */
  public void ejbCreate(String ID) {

    ShopperHome shopperHome = null; // Shopper Home interface
    AccountHome accountHome = null; // Account Home interface
    Context ic = null;
    Object lookupObject = null;

    try {
          SimpleTrace.trace("FacadeBean.ejbCreate : Acquiring InitialContext ...");
          ic = new InitialContext();
    } catch(NamingException ne) {
          SimpleTrace.trace("InitialContext could not be acquired: " + ne);
          throw new EJBException("InitialContext could not be acquired: " + ne);
    }

    try {
          // Shopper
          SimpleTrace.trace("FacadeBean.ejbCreate : looking up ["+JndiNames.SHOPPER+"]...");
          lookupObject = ic.lookup(JndiNames.SHOPPER);
          SimpleTrace.trace("FacadeBean.ejbCreate : before narrowing of ["+lookupObject.getClass().getName()+"]...");
          shopperHome  = (ShopperHome)javax.rmi.PortableRemoteObject.narrow(lookupObject,ShopperHome.class);
          SimpleTrace.trace("FacadeBean.ejbCreate : shopperHome ["+shopperHome.getClass().getName()+"]");

          // Account
          SimpleTrace.trace("FacadeBean.ejbCreate : looking up ["+JndiNames.ACCOUNT+"]...");
          lookupObject = ic.lookup(JndiNames.ACCOUNT);
          SimpleTrace.trace("FacadeBean.ejbCreate : before narrowing of ["+lookupObject.getClass().getName()+"]...");
          accountHome  = (AccountHome)javax.rmi.PortableRemoteObject.narrow(lookupObject, AccountHome.class);
          SimpleTrace.trace("FacadeBean.ejbCreate : accountHome ["+accountHome.getClass().getName()+"]");
    }
    catch (NamingException ne) {
            SimpleTrace.trace(" # FacadeBean.ejbCreate :  NamingException : " + ne);
            throw new EJBException(" # FacadeBean.ejbCreate :  NamingException : " + ne);
    }

    try {
            // Log.getInstance().traceInfo(Log.APP_BUSINESS,"create StatefulShopperBean ID["+ID+"]...");
            this.shopper = shopperHome.create(ID);
            SimpleTrace.trace("FacadeBean.ejbCreate : create Shopper done !");

    } catch (CreateException e) {
            SimpleTrace.trace("# FacadeBean.ejbCreate : CreateException on SHOPPER : " + e);
            throw new EJBException("# FacadeBean.ejbCreate : CreateException SHOPPER :" + e);
    } catch (java.rmi.RemoteException e) {
            SimpleTrace.trace("# FacadeBean.ejbCreate : RemoteException SHOPPER : " + e);
            throw new EJBException("# FacadeBean.ejbCreate : RemoteException SHOPPER : " + e);
    }

    try {
            System.out.println("FacadeBean.ejbCreate : findByPrimaryKey Account ["+ID+"]...");
            this.account = accountHome.findByPrimaryKey(new AccountPK(ID));
            System.out.println("FacadeBean.ejbCreate : findByPrimaryKey Account done !");

    } catch (FinderException e) {
            System.out.println("# FacadeBean.ejbCreate : FinderException on ACCOUNT : " + e);
            throw new EJBException("# FacadeBean.ejbCreate : FinderException on ACCOUNT :" + e);
    } catch (java.rmi.RemoteException e) {
            System.out.println("# FacadeBean.ejbCreate : RemoteException on ACCOUNT : " + e);
            throw new EJBException("# FacadeBean.ejbCreate : RemoteException on ACCOUNT : " + e);
    }

    System.out.println("FacadeBean.ejbCreate : SUCCESFULLY done !\n");

  }

  /****************************** Session Steful Shooper Facade ****************************/

  /** Add : Facade to addItem of StatefulShopperBean */
  /**
   * Pass to Shopper the prod received
   * @param prod the prod to buy
   * @throws BusinessException
   */
  public void addItem(ProductOM prod) throws BusinessException{
    	SimpleTrace.trace("FacadeBean.addItem called ...");
        try{
            double totToPay = prod.getPrice()+ shopper.getAmount();
            if( totToPay > account.getBalance())
               throw new BusinessException("FacadeBean.addItem : product ["+prod+"] NOT added because total price to pay["+totToPay+"] is major than ["+this.getBalance()+"] !");

	    shopper.addItem(prod);
            return;
         }catch(RemoteException e) {  // RemoteException
            e.printStackTrace();
            throw new BusinessException(e);
        }
  }

  /** Remove : Facade to removeItem of StatefulShopperBean */
  public void removeItem(ProductOM prod)throws BusinessException {
    	SimpleTrace.trace("FacadeBean.removeItem called ...");
        try{
	  shopper.removeItem(prod);
            return;
         }catch(Exception e) {  // RemoteException, ECommerceException
            e.printStackTrace();
            throw new BusinessException(e);
        }
  }

  /** getAmount : facade to getAmount of StatefulShopperBean */
  public double getAmount() throws BusinessException{
        try {
            return shopper.getAmount();
        }catch(RemoteException re) {
            re.printStackTrace();
            throw new BusinessException(re);
        }
  }

  public int getBasketSize() throws RemoteException{
  	System.out.println(new java.util.Date().toString() + "FacadeBean.getBasketSize() : return["+this.shopper.getBasketSize()+"]...");
    	return this.shopper.getBasketSize();
  }

  /**
    * Obtains the customer name from the customer entity bean
    * @return The customer name
    */
    public String getName() {
        System.out.println("FacadeBean getName called ...");

        try {
            return account.getName();
        } catch (java.rmi.RemoteException e) {
            System.out.println("FacadeBean getName() failed: " + e);
            throw new EJBException("FacadeBean getName() failed: " + e);
        }
    }

    /**
     * Obtains the account balance from the account entity bean
     * @return The current balance in the account
     */
    public double getBalance() {
        System.out.println("FacadeBean.getBalance called ...");
        try {
            return account.getBalance();
        } catch (java.rmi.RemoteException e) {
            System.out.println("FacadeBean getBalance() failed: " + e);
            throw new EJBException("FacadeBean getBalance() failed: " + e);
        }
    }

    /**
     *
     * @return
     * @throws BusinessException
     */
     public String getEmail() {
        System.out.println("FacadeBean.getEmail called ...");
        try {
            return account.getEmail();
        } catch (java.rmi.RemoteException e) {
            System.out.println("FacadeBean getEmail failed: " + e);
            throw new EJBException("FacadeBean getEmail failed: " + e);
        }
    }


  /**
   * getAccount
   * @return the Object Value populated by Entity's values
   * @throws BusinessException
   * @throws RemoteException
   */
   public AccountOM getAccount() throws BusinessException {

        try{
        /*
          AccountOM objectValue = new AccountOM("-1",
                                                this.account.getName(),
                                                this.account.getEmail(),
                                                this.account.getBalance());
        */
          AccountOM objectValue = this.account.getAccount();
          SimpleTrace.trace("FacadeBean.getAccount : returning ["+objectValue+"]...");
          return objectValue;

        } catch (java.rmi.RemoteException e) {
            SimpleTrace.trace("FacadeBean getAccount() failed: " + e);
            throw new BusinessException("FacadeBean getAccount() failed: " + e);
        }
    }

    /**
     * Withdraws <code>amount</code> funds from the account.
     * @param amount The amount of money to withdraw
     * @exception BankException Thrown if withdrawing <code>amount</code>
     *   would result in a negative balance, or <code>amount</code> is
     *   negetive.
     */
    public void withdraw(double amount) throws BusinessException{
        SimpleTrace.trace("FacadeBean withdraw called ...");

        try {
            if (account.getBalance() < amount) throw new BusinessException("Insufficient funds on withdraw");
            account.setBalance(account.getBalance() - amount);
        } catch (java.rmi.RemoteException e) {
            SimpleTrace.trace("FacadeBean withdraw failed: " + e);
            throw new BusinessException(e);
        }
    }

    /**
     * Deposits <code>amount</code> funds in to the account.
     * @param amount The amount of money to deposit
     * @exception BankException Thrown if <code>amount</code> passed was negative
     */
    public void deposit(double amount) throws BusinessException {
        SimpleTrace.trace("FacadeBean deposit called");
        try {
            if (amount < 0)
                throw new BusinessException("Deposit of ["+amount+"] NOT allowed !");
            account.setBalance(account.getBalance() + amount);
        } catch (java.rmi.RemoteException e) {
            SimpleTrace.trace("FacadeBean deposit failed: " + e);
            throw new BusinessException(e);
        }
    }

    //*************************** EJB Methods Start Here *****************************

    /** A no-op, as all state is restored by the container automatically */
    public void ejbActivate() {
        SimpleTrace.trace("FacadeBean.ejbActivate called");
    }


    /** A no-op, as all instance variables are serializable */
    public void ejbPassivate() {
        SimpleTrace.trace("FacadeShopperBean.ejbPassivate called");
    }

    /**
     * Stores the session context passed in to an instance variable <code>ctx</code>.
     * @param ctx The session context asscociated with the bean.
     */
    public void setSessionContext(SessionContext ctx) {
        SimpleTrace.trace("FacadeBean.setSessionContext called");
        this.ctx = ctx;
    }


    /**
     * A no-op, as when the container removes the bean, all the account data is removed for us.
     * We don't want to call ejbRemove on the entity beans as this would delete the account from
     * the database.
     */
    public void ejbRemove() {
        SimpleTrace.trace("FacadeBean.ejbRemove called");
    }
}
